/*
 * $Id: OldJWNLDistances.java,v 1.2 2004/10/18 11:25:29 euzenat Exp $
 *
 * Copyright (C) INRIA Rhne-Alpes, 2003-2004
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation; either version 2.1 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA
 */

package fr.inrialpes.exmo.align.ling;

import fr.inrialpes.exmo.align.impl.method.StringDistances;

import java.io.FileInputStream;
import java.util.Enumeration;
import java.util.Hashtable;
import java.util.Iterator;
import java.util.List;
import java.util.Vector;

import net.didion.jwnl.JWNL;
import net.didion.jwnl.JWNLException;
import net.didion.jwnl.data.IndexWord;
import net.didion.jwnl.data.POS;
import net.didion.jwnl.data.PointerType;
import net.didion.jwnl.data.Synset;
import net.didion.jwnl.dictionary.Dictionary;
import net.didion.jwnl.data.PointerUtils;
import net.didion.jwnl.data.list.PointerTargetNode;
import net.didion.jwnl.data.list.PointerTargetNodeList;
import net.didion.jwnl.data.list.PointerTargetTree;
import net.didion.jwnl.data.relationship.AsymmetricRelationship;
import net.didion.jwnl.data.relationship.Relationship;
import net.didion.jwnl.data.relationship.SymmetricRelationship;

/**
 * Compute a string distance using the JWNL API (WordNet API)
 * 
 * @author Jerome Pierson, David Loup
 * @version $Id: JWNLDistances.java,v 1.0 2004/08/04
 *
 */
public class OldJWNLDistances {

	public static final double NOUN_WEIGHT = 0.60;
	public static final double ADJ_WEIGHT = 0.25;
	public static final double VERB_WEIGHT = 0.15;
	
	/**
	 * Initialize the JWNL API. Must be done one time before computing distance
	 * Need to configure the file_properties.xml located in the current
	 * directory (ontoalign)
	 *  
	 */
	public void Initialize() {
		try {
			JWNL.initialize(new FileInputStream("./file_properties.xml"));
		} catch (Exception ex) {
			ex.printStackTrace();
			System.exit(-1);
		}
	}

	/**
	 * Compute a basic distance between 2 strings using WordNet synonym.
	 * 
	 * @param s1
	 * @param s2
	 * @return Distance between s1 & s2 (return 1 if s2 is a synonym of s1, else
	 *         return a BasicStringDistance between s1 & s2)
	 */
	public double BasicSynonymDistance(String s1, String s2) {
		double Dist = 0.0;
		double Dists1s2;
		int i, j, k = 0;
		int synonymNb = 0;
		int besti = 0, bestj = 0;
		int syno = 0;
		double DistTab[];
		IndexWord index = null;
		Synset Syno[] = null;

		// Modification 09/13/2004 by David Loup
		s1 = s1.toUpperCase();
		s2 = s2.toUpperCase();
		
		Dists1s2 = StringDistances.subStringDistance(s1, s2);

		try {
			// Lookup for first string
			index = Dictionary.getInstance().lookupIndexWord(POS.NOUN, s1);
		} catch (Exception ex) {
			ex.printStackTrace();
			System.exit(-1);
		}
		// if found in the dictionary
		if (index != null) {
			try {
				// get the groups of synonyms for each sense
				Syno = index.getSenses();
			} catch (JWNLException e) {
				e.printStackTrace();
			}
			// number of senses for the word s1
			synonymNb = index.getSenseCount();
			DistTab = new double[synonymNb];
			// for each sense
			for (k = 0; k < synonymNb; k++) {
				// for each synonym of this sense
				for (j = 0; j < Syno[k].getWordsSize(); j++) {
					Dist = StringDistances.subStringDistance(Syno[k].getWord(j).getLemma(), s2);
					if (Dist < Dists1s2) {
						Dists1s2 = Dist;
						besti = k;
						bestj = j;
					}
				}
			}
		}
		
		//System.err.println(s1 + " - " + s2 + " = " + Dists1s2);
		return Dists1s2;
	}
	
	public double BasicSynonymDistanceNew(String s1, String s2) {
		double Dist = 0.0;
		double Dists1s2;
		int i, j, k = 0;
		int synonymNb = 0;
		int besti = 0, bestj = 0;
		int syno = 0;
		double DistTab[];
		IndexWord index = null;
		Synset Syno1[], Syno2[] = null;

		//s1 = s1.toUpperCase();
		//s2 = s2.toUpperCase();
		
		Dists1s2 = StringDistances.subStringDistance(s1, s2);
		
		//if (Dists1s2 == 0) return Dists1s2;
		//else {
			
			if (s1.equals(s1.toUpperCase()) || s1.equals(s1.toLowerCase())) {
				try {
					// Lookup for first string
					index = Dictionary.getInstance().lookupIndexWord(POS.NOUN, s1);
				} catch (Exception ex) {
					ex.printStackTrace();
					System.exit(-1);
				}
				// if not found in the dictionary
				if (index == null) return Dists1s2;
				else {
					double dist = computeBestMatch(s1, s2);
				}
			}
			else {
				double dist = computeBestMatch(s1, s2);
			}
		//}
		return Dists1s2;
	}

	public double computeBestMatch(String s1, String s2) {
		Vector s1Tokens = new Vector();
		Vector s2Tokens = new Vector();
		tokenizes(s1,s1Tokens);
		tokenizes(s2,s2Tokens);
		
		// tokens storage
		Vector vg;
		Vector vp;
		vg = s1Tokens.size() >= s2Tokens.size() ? s1Tokens : s2Tokens;
		vp = s1Tokens.size() >= s2Tokens.size() ? s2Tokens : s1Tokens;
		
		double dist = 0;
		
		// tokens depending on their nature
		Hashtable nouns1 = new Hashtable();
		Hashtable adjectives1 = new Hashtable();
		Hashtable verbs1 = new Hashtable();
		
		Hashtable nouns2 = new Hashtable();
		Hashtable adjectives2 = new Hashtable();
		Hashtable verbs2 = new Hashtable();
		
		// Put tokens in the right category		
		Iterator gIt = vg.iterator();
		while (gIt.hasNext()) {
			lookUpWord((String) gIt.next(), nouns1, adjectives1, verbs1);
		}
		Iterator pIt = vp.iterator();
		while (pIt.hasNext()) {
			lookUpWord((String) pIt.next(), nouns2, adjectives2, verbs2);
		}
		
		Enumeration nounsEnum1 = nouns1.elements();
		while (nounsEnum1.hasMoreElements()) {
			IndexWord index1 = (IndexWord) nounsEnum1.nextElement();
			Enumeration nounsEnum2 = nouns2.elements();
			while (nounsEnum2.hasMoreElements()) {
				IndexWord index2 = (IndexWord) nounsEnum2.nextElement();
				System.out.println(findMatchForTokens(index1, index2));
			}
		}
		

//		System.out.println(nouns1);
//		System.out.println(adjectives1);
//		System.out.println(verbs1);
//		System.out.println(nouns2);
//		System.out.println(adjectives2);
//		System.out.println(verbs2);
		
		return dist;
	}
	

	public double findMatchForTokens(IndexWord index1, IndexWord index2) {
		// the max number of common concepts between the two tokens
		int maxCommon = 0;
		
		// the two lists giving the best match
		PointerTargetNodeList best1 = null;
		PointerTargetNodeList best2 = null;
		
		// the two lists currently compared
		PointerTargetNodeList ptnl1 = null;
		PointerTargetNodeList ptnl2 = null;
					
		if (index1 != null && index2 != null) {
			// The two tokens existe in WordNet, we find the "depth"
			try {
				// Synsets for each token
				Synset[] Syno1 = index1.getSenses();
				Synset[] Syno2 = index2.getSenses();
				for (int i = 0; i < index1.getSenseCount(); i++) {
					
					Synset synset1 = Syno1[i];
					for (int k = 0; k < index2.getSenseCount(); k++) {
						
						// Best match between current lists
						int maxBetweenLists = 0;
						
						Synset synset2 = Syno2[k];
						
						List hypernymList1 = PointerUtils.getInstance().getHypernymTree(synset1).toList();
						List hypernymList2 = PointerUtils.getInstance().getHypernymTree(synset2).toList();
														
						Iterator list1It = hypernymList1.iterator();
						// browse lists
						while (list1It.hasNext()) {
							ptnl1 = (PointerTargetNodeList) list1It.next();
							Iterator list2It = hypernymList2.iterator();
							while (list2It.hasNext()) {
								ptnl2 = (PointerTargetNodeList) list2It.next();

								int cc = getCommonConcepts(ptnl1, ptnl2);
								if (cc > maxBetweenLists) {
									maxBetweenLists = cc;
									best1 = ptnl1;
									best2 = ptnl2;
								}
							}
						}
						if (maxBetweenLists > maxCommon) {
							maxCommon = maxBetweenLists;
						}
						
					}
				
				}
				System.out.println("common = "+maxCommon);
				System.out.println("value = "+( (2*maxCommon) / (best1.size() + best2.size())));
				if (best1 != null) best1.print();
				System.out.println("---------------------------------");
				if (best2 != null) best2.print();
				return (2*maxCommon / (best1.size() + best2.size()));
			}
			catch (JWNLException je) {
				je.printStackTrace();
				System.exit(-1);
			}
		}
		return 0;
	}
	
	public void tokenizes(String s, Vector sTokens) {
		String str1 = s;
		// starts on the second character of the string
		int start = 0;
		int car = start+1;
		while (car < str1.length()) {
				while (car < str1.length() && !(str1.charAt(car) < 'Z')) {
					car++;
				}
				sTokens.add(str1.substring(start,car));
				start = car;
				car = start + 1;
		}
	}
	
	/**
	 * TODO Look up for other things than nouns 
	 * @param word
	 * @return
	 */
	public IndexWord lookUpWord(String word, Hashtable nouns, Hashtable adjectives, Hashtable verbs) {
		IndexWord index = null;
		try {
			// Lookup for word in adjectives
			index = Dictionary.getInstance().lookupIndexWord(POS.ADJECTIVE, word );
	
			if (index == null) {
				// Lookup for word in nouns
				index = Dictionary.getInstance().lookupIndexWord(POS.NOUN, word );
			}
			else {
				System.out.println(word+" is an adjective");
				adjectives.put(word,index);
				return index;
			}
			if (index == null) {
				// Lookup for word in verbs
				index = Dictionary.getInstance().lookupIndexWord(POS.VERB, word );
			}
			else {
				System.out.println(word+" is a noun");
				nouns.put(word,index);
				return index;
			}
			if (index != null) {
				System.out.println(word+" is a verb");
				verbs.put(word,index);
				return index;
			}
			
		}
		catch (Exception ex) {
			ex.printStackTrace();
			System.exit(-1);
		}
		return index;
	}
	
	public void display(Synset syn) {
		String str = "";
		for (int s=0; s<syn.getWordsSize(); s++) {
			str += syn.getWord(s);
		}
		System.out.println(str);
	}
	
	public int getCommonConcepts(PointerTargetNodeList list1, PointerTargetNodeList list2) {
		int cc = 0;
		int i = 1;
		while (i <= Math.min(list1.size(),list2.size()) &&((PointerTargetNode) list1.get(list1.size()-i)).getSynset() == ((PointerTargetNode) list2.get(list2.size()-i)).getSynset()) {
			cc++;
			i++;
		}
		return cc;
	}
	
	public static void main(String[] args) {
		Vector v = new Vector();
		OldJWNLDistances j = new OldJWNLDistances();
		j.Initialize();
		String s1 = "isOldBook";
		String s2 = "hasAncientTome";
		j.BasicSynonymDistanceNew(s1,s2);
	}
	
	
	
	
	
	
	
	
	
	
	
	/**
	 * Compute a distance between 2 strings using WordNet synonym Get all the
	 * synonym for s1 , syno1 Get all the synonym for s2, syno2 compute
	 * intersection syno1(inter)syno2, I compute union syno1(union)syno2, U The
	 * distance = card I /card U
	 * 
	 * @param s1
	 * @param s2
	 * @return Distance between s1 & s2 = card I /card U
	 */

	public double SynonymDistance(String s1, String s2) {
		double result = 0.000000;
		int cardI = 0, cardU = 0;
		int i = 0, j = 0, k = 0;
		String Syno1[];
		String Syno2[];
		String I[] = null;
		String U[] = null;
		int nbmot1;
		int nbmot2;

		/** Build the list of synonym for s1 & s2 * */
		nbmot1 = CountMaxSensSyno(s1);
		nbmot2 = CountMaxSensSyno(s2);
		Syno1 = new String[nbmot1];
		Syno2 = new String[nbmot2];
		Syno1 = GetAllSyno(s1);
		Syno2 = GetAllSyno(s2);
		/** Compute intersection between the list of synonym : I* */
		I = Inter(Syno1, Syno2);
		/** Compute union between the list of synonym U * */
		U = Union(Syno1, Syno2);
		/** compute card I / card U * */
		for (i = 0; i < I.length; i++) {
			if (I[i] != null) {
				cardI++;
			}
		}
		for (i = 0; i < U.length; i++) {
			if (U[i] != null) {
				cardU++;
			}
		}
		if (cardU != 0) {
			result = (double) cardI / (double) cardU;
		}
		return (result);
	}

	/**
	 * Compute a distance between 2 strings using WordNet synonym, hypernym,
	 * etc... Get all the ... for s1 , syno1 Get all the ... for s2, syno2
	 * compute intersection syno1(inter)syno2, I compute union
	 * syno1(union)syno2, U The distance = card I /card U NOT FINISHED YET!
	 * 
	 * @param s1
	 * @param s2
	 * @return The Distance between s1 & s2
	 */

	public double CompleteDistance(String s1, String s2) {
		double result = 0.000000;
		int cardI = 0, cardU = 0;
		int i = 0, j = 0, k = 0;
		String Syno1[];
		String Syno2[];
		String I[] = null;
		String U[] = null;
		int nbmot1;
		int nbmot2;

		/** Build the list of related word for s1 & s2 * */
		nbmot1 = CountMaxSens(s1);
		nbmot2 = CountMaxSens(s2);
		Syno1 = new String[nbmot1];
		Syno2 = new String[nbmot2];
		Syno1 = GetAll(s1);
		Syno2 = GetAll(s2);

		/** Compute intersection between the list of synonym : I* */
		I = Inter(Syno1, Syno2);
		/** Compute union between the list of synonym U * */
		U = Union(Syno1, Syno2);
		/** compute card I / card U * */
		for (i = 0; i < I.length; i++) {
			if (I[i] != null) {
				cardI++;
			}
		}
		for (i = 0; i < U.length; i++) {
			if (U[i] != null) {
				cardU++;
			}
		}
		if (cardU != 0) {
			result = (double) cardI / (double) cardU;
		}
		return (result);
	}

	/**
	 * Count the maximum number of sens for a word using all related word NOT
	 * FINISHED YET!
	 * 
	 * @param S
	 * @return Number of sens for the word s
	 */
	private int CountMaxSens(String S) {
		IndexWord index = null;
		Synset S1[] = null;
		int i = 0;
		int sens1 = 0;
		int nbmot1 = 0;
		//PointerUtils utilitaire;
		PointerTargetNodeList pointeur = null;

		try {
			index = Dictionary.getInstance().lookupIndexWord(POS.NOUN, S);
		} catch (Exception ex) {
			ex.printStackTrace();
			System.exit(-1);
		}
		if (index != null) {
			try {
				S1 = index.getSenses();
				sens1 = index.getSenseCount();
			} catch (JWNLException e) {
				e.printStackTrace();
			}

		}
		for (i = 0; i < sens1; i++) {
			nbmot1 = nbmot1 + S1[i].getWordsSize();
			try {

				pointeur = PointerUtils.getInstance().getDirectHypernyms(S1[i]);
				//System.out.println(pointeur.toString());
			} catch (JWNLException e) {
			}

		}

		try {
			index = Dictionary.getInstance().lookupIndexWord(POS.VERB, S);
		} catch (Exception ex) {
			ex.printStackTrace();
			System.exit(-1);
		}
		if (index != null) {
			try {
				S1 = index.getSenses();
				sens1 = index.getSenseCount();
			} catch (JWNLException e) {
				e.printStackTrace();
			}
		}
		for (i = 0; i < sens1; i++) {
			nbmot1 = nbmot1 + S1[i].getWordsSize();
		}

		try {
			index = Dictionary.getInstance().lookupIndexWord(POS.ADJECTIVE, S);
		} catch (Exception ex) {
			ex.printStackTrace();
			System.exit(-1);
		}
		if (index != null) {
			try {
				S1 = index.getSenses();
				sens1 = index.getSenseCount();
			} catch (JWNLException e) {
				e.printStackTrace();
			}
		}
		for (i = 0; i < sens1; i++) {
			nbmot1 = nbmot1 + S1[i].getWordsSize();
		}

		try {
			index = Dictionary.getInstance().lookupIndexWord(POS.ADVERB, S);
		} catch (Exception ex) {
			ex.printStackTrace();
			System.exit(-1);
		}
		if (index != null) {
			try {
				S1 = index.getSenses();
				sens1 = index.getSenseCount();
			} catch (JWNLException e) {
				e.printStackTrace();
			}
		}
		for (i = 0; i < sens1; i++) {
			nbmot1 = nbmot1 + S1[i].getWordsSize();
		}

		return (nbmot1);

	}

	/**
	 * Count the maximum number of sens for a word, using synonym relation
	 * 
	 * @param S
	 * @return Number of sens for the word s using synonym
	 */
	private int CountMaxSensSyno(String S) {
		IndexWord index = null;
		Synset S1[] = null;
		int i = 0;
		int sens1 = 0;
		int nbmot1 = 0;
		//Counting NOUN
		try {
			index = Dictionary.getInstance().lookupIndexWord(POS.NOUN, S);
		} catch (Exception ex) {
			ex.printStackTrace();
			System.exit(-1);
		}
		if (index != null) {
			try {
				S1 = index.getSenses();
				sens1 = index.getSenseCount();
			} catch (JWNLException e) {
				e.printStackTrace();
			}
		}
		for (i = 0; i < sens1; i++) {
			nbmot1 = nbmot1 + S1[i].getWordsSize();
		}
		// Counting VERB
		try {
			index = Dictionary.getInstance().lookupIndexWord(POS.VERB, S);
		} catch (Exception ex) {
			ex.printStackTrace();
			System.exit(-1);
		}
		if (index != null) {
			try {
				S1 = index.getSenses();
				sens1 = index.getSenseCount();
			} catch (JWNLException e) {
				e.printStackTrace();
			}
		}
		for (i = 0; i < sens1; i++) {
			nbmot1 = nbmot1 + S1[i].getWordsSize();
		}
		//Counting ADJECTIVE
		try {
			index = Dictionary.getInstance().lookupIndexWord(POS.ADJECTIVE, S);
		} catch (Exception ex) {
			ex.printStackTrace();
			System.exit(-1);
		}
		if (index != null) {
			try {
				S1 = index.getSenses();
				sens1 = index.getSenseCount();
			} catch (JWNLException e) {
				e.printStackTrace();
			}
		}
		for (i = 0; i < sens1; i++) {
			nbmot1 = nbmot1 + S1[i].getWordsSize();
		}
		//Counting ADVERB
		try {
			index = Dictionary.getInstance().lookupIndexWord(POS.ADVERB, S);
		} catch (Exception ex) {
			ex.printStackTrace();
			System.exit(-1);
		}
		if (index != null) {
			try {
				S1 = index.getSenses();
				sens1 = index.getSenseCount();
			} catch (JWNLException e) {
				e.printStackTrace();
			}
		}
		for (i = 0; i < sens1; i++) {
			nbmot1 = nbmot1 + S1[i].getWordsSize();
		}

		return (nbmot1);

	}

	/**
	 * Search all the synonym for a word
	 * 
	 * @param S
	 * @return The tab with all the synonym of the word s
	 */
	private String[] GetAllSyno(String S) {
		String Syno[] = null;
		IndexWord index = null;
		Synset[] Synso = null;
		int sens = 0;
		int nbmot = 0;
		int i = 0, j = 0, k = 0, l = 0;
		boolean trouve = false;
		int strcomp = 0;

		nbmot = CountMaxSensSyno(S);
		Syno = new String[nbmot];
		//Looking for word as a NOUN
		try {
			index = Dictionary.getInstance().lookupIndexWord(POS.NOUN, S);
		} catch (Exception ex) {
			ex.printStackTrace();
			System.exit(-1);
		}
		if (index != null) {
			try {
				Synso = index.getSenses();
				sens = index.getSenseCount();
			} catch (JWNLException e) {
				e.printStackTrace();
			}
		}

		for (i = 0; i < sens; i++) {
			for (j = 0; j < Synso[i].getWordsSize(); j++) {
				for (l = 0; l < k; l++) {
					if (Synso[i].getWord(j).getLemma().compareTo(Syno[l]) == 0) {
						trouve = true;
					}
				}
				if (trouve != true) {
					Syno[k++] = Synso[i].getWord(j).getLemma();
				}
				trouve = false;
			}
		}
		//	Looking for word as a VERB
		try {
			index = Dictionary.getInstance().lookupIndexWord(POS.VERB, S);
		} catch (Exception ex) {
			ex.printStackTrace();
			System.exit(-1);
		}
		if (index != null) {
			try {
				Synso = index.getSenses();
				sens = index.getSenseCount();
			} catch (JWNLException e) {
				e.printStackTrace();
			}
		}

		for (i = 0; i < sens; i++) {
			for (j = 0; j < Synso[i].getWordsSize(); j++) {
				for (l = 0; l < k; l++) {
					if (Synso[i].getWord(j).getLemma().compareTo(Syno[l]) == 0) {
						trouve = true;
					}
				}
				if (trouve != true) {
					Syno[k++] = Synso[i].getWord(j).getLemma();
				}
				trouve = false;
			}
		}
		//	Looking for word as a AJECTIVE
		try {
			index = Dictionary.getInstance().lookupIndexWord(POS.ADJECTIVE, S);
		} catch (Exception ex) {
			ex.printStackTrace();
			System.exit(-1);
		}
		if (index != null) {
			try {
				Synso = index.getSenses();
				sens = index.getSenseCount();
			} catch (JWNLException e) {
				e.printStackTrace();
			}
		}

		for (i = 0; i < sens; i++) {
			for (j = 0; j < Synso[i].getWordsSize(); j++) {
				for (l = 0; l < k; l++) {
					if (Synso[i].getWord(j).getLemma().compareTo(Syno[l]) == 0) {
						trouve = true;
					}
				}
				if (trouve != true) {
					Syno[k++] = Synso[i].getWord(j).getLemma();
				}
				trouve = false;
			}
		}
		//	Looking for word as a ADVERB
		try {
			index = Dictionary.getInstance().lookupIndexWord(POS.ADVERB, S);
		} catch (Exception ex) {
			ex.printStackTrace();
			System.exit(-1);
		}
		if (index != null) {
			try {
				Synso = index.getSenses();
				sens = index.getSenseCount();
			} catch (JWNLException e) {
				e.printStackTrace();
			}
		}
		for (i = 0; i < sens; i++) {
			for (j = 0; j < Synso[i].getWordsSize(); j++) {
				for (l = 0; l < k; l++) {
					if (Synso[i].getWord(j).getLemma().compareTo(Syno[l]) == 0) {
						trouve = true;
					}
				}
				if (trouve != true) {
					Syno[k++] = Synso[i].getWord(j).getLemma();
				}
				trouve = false;
			}
		}
		return (Syno);
	}

	/**
	 * Search for ALL the related word for a given word NOT FINISHED YET!
	 * 
	 * @param S
	 * @return The tab with all the related word of the word s
	 */
	private String[] GetAll(String S) {
		String Syno[] = null;
		IndexWord index = null;
		Synset[] Synso = null;
		int sens = 0;
		int nbmot = 0;
		int i = 0, j = 0, k = 0, l = 0;
		boolean trouve = false;
		int strcomp = 0;

		nbmot = CountMaxSens(S);
		Syno = new String[nbmot];

		try {
			index = Dictionary.getInstance().lookupIndexWord(POS.NOUN, S);
		} catch (Exception ex) {
			ex.printStackTrace();
			System.exit(-1);
		}
		if (index != null) {
			try {
				Synso = index.getSenses();
				sens = index.getSenseCount();
			} catch (JWNLException e) {
				e.printStackTrace();
			}
		}

		for (i = 0; i < sens; i++) {
			for (j = 0; j < Synso[i].getWordsSize(); j++) {
				for (l = 0; l < k; l++) {
					if (Synso[i].getWord(j).getLemma().compareTo(Syno[l]) == 0) {
						trouve = true;
					}
				}
				if (trouve != true) {
					Syno[k++] = Synso[i].getWord(j).getLemma();
				}
				trouve = false;
			}
		}
		try {
			index = Dictionary.getInstance().lookupIndexWord(POS.VERB, S);
		} catch (Exception ex) {
			ex.printStackTrace();
			System.exit(-1);
		}
		if (index != null) {
			try {
				Synso = index.getSenses();
				sens = index.getSenseCount();
			} catch (JWNLException e) {
				e.printStackTrace();
			}
		}

		for (i = 0; i < sens; i++) {
			for (j = 0; j < Synso[i].getWordsSize(); j++) {
				for (l = 0; l < k; l++) {
					if (Synso[i].getWord(j).getLemma().compareTo(Syno[l]) == 0) {
						trouve = true;
					}
				}
				if (trouve != true) {
					Syno[k++] = Synso[i].getWord(j).getLemma();
				}
				trouve = false;
			}
		}

		try {
			index = Dictionary.getInstance().lookupIndexWord(POS.ADJECTIVE, S);
		} catch (Exception ex) {
			ex.printStackTrace();
			System.exit(-1);
		}
		if (index != null) {
			try {
				Synso = index.getSenses();
				sens = index.getSenseCount();
			} catch (JWNLException e) {
				e.printStackTrace();
			}
		}

		for (i = 0; i < sens; i++) {
			for (j = 0; j < Synso[i].getWordsSize(); j++) {
				for (l = 0; l < k; l++) {
					if (Synso[i].getWord(j).getLemma().compareTo(Syno[l]) == 0) {
						trouve = true;
					}
				}
				if (trouve != true) {
					Syno[k++] = Synso[i].getWord(j).getLemma();
				}
				trouve = false;
			}
		}

		try {
			index = Dictionary.getInstance().lookupIndexWord(POS.ADVERB, S);
		} catch (Exception ex) {
			ex.printStackTrace();
			System.exit(-1);
		}
		if (index != null) {
			try {
				Synso = index.getSenses();
				sens = index.getSenseCount();
			} catch (JWNLException e) {
				e.printStackTrace();
			}
		}
		for (i = 0; i < sens; i++) {
			for (j = 0; j < Synso[i].getWordsSize(); j++) {
				for (l = 0; l < k; l++) {
					if (Synso[i].getWord(j).getLemma().compareTo(Syno[l]) == 0) {
						trouve = true;
					}
				}
				if (trouve != true) {
					Syno[k++] = Synso[i].getWord(j).getLemma();
				}
				trouve = false;
			}
		}
		return (Syno);
	}

	/**
	 * Compute the intersection between two group of words
	 * @param S1[] first group
	 * @param S2[] second group
	 * @return a string[] with the intersection of S1[] and S[]
	 * 
	 * */

	private String[] Inter(String[] S1, String[] S2) {
		String result[] = null;
		int i = 0, j = 0, k = 0, l = 0;
		boolean trouve = false;

		i = S1.length;
		j = S2.length;
		if (i < j) {
			result = new String[j];
		} else
			result = new String[i];

		for (i = 0; i < S1.length; i++) {
			for (j = 0; j < S2.length; j++) {
				if (S1[i] != null && S2[j] != null) {
					if (S1[i].compareTo(S2[j]) == 0) {
						trouve = true;
					}
				}
			}
			if (trouve == true) {
				result[k++] = S1[i];
			}
			trouve = false;
		}
		return (result);
	}

	/**
	 * Compute the Union between two group of words
	 * @param S1[] first group
	 * @param S2[] second group
	 * @return String[] with the Union of S1[] and S2[]
	 */

	private String[] Union(String[] S1, String[] S2) {
		String result[] = null;
		int i = 0, j = 0, k = 0, l = 0;
		boolean trouve = false;

		i = S1.length;
		j = S2.length;
		result = new String[i + j];

		for (i = 0; i < S1.length; i++) {
			result[k++] = S1[i];
		}
		for (j = 0; j < S2.length; j++) {
			for (l = 0; l < S1.length; l++) {

				if (S1[l] != null && S2[j] != null) {
					if (S2[j].compareTo(S1[l]) == 0) {
						trouve = true;
					}
				}
			}
			if (trouve == false) {
				result[k++] = S2[j];
			}
			trouve = false;
		}
		return (result);
	}
	
	
	

}
